import 'package:billettapp_for_vitensenteret/resources/structs.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:image_picker/image_picker.dart';
import 'firebase_functions.dart';

//TODO: When a user who previously had a yearly membership buys a new one, refresh that one
//TODO: Make it possible to delete users
//TODO: Make it possible to add profile picture to new users
//TODO: Delete all expired tickets over a certain age

List<UserData> adminUserData = allUserData;
List<TicketData> adminUserTickets = allUserTickets;

///Ticket page for admins or employees at Vitensenteret
/// Shows all users and all tickets
class AdminTicketPage extends StatefulWidget {
  const AdminTicketPage({super.key});

  @override
  State<AdminTicketPage> createState() => _AdminTicketPageState();
}

class _AdminTicketPageState extends State<AdminTicketPage> {
  late TextEditingController _searchController;
  final GlobalKey<FormState> _formKey = GlobalKey<FormState>();
  final ImagePicker _picker = ImagePicker();

  @override
  void initState() {
    super.initState();
    _searchController = TextEditingController();
  }

  @override
  void dispose() {
    _searchController.dispose();
    super.dispose();
  }

  /// Sort [adminUserTickets] and [adminUserData] by either date or alphabetically based on [type]
  sortBy(String type) {
    if (type == 'Dato') {
      adminUserTickets.sort((a, b) => int.parse(a.purchaseDate).compareTo(int.parse(b.purchaseDate)));
      adminUserData.sort((a, b) => a.birthday.compareTo(b.birthday));
    }
    if (type == 'Alfabetisk') {
      adminUserTickets.sort((a, b) => a.activatedBy.toLowerCase().compareTo(b.activatedBy.toLowerCase()));
      adminUserData.sort((a, b) => a.name.toLowerCase().compareTo(b.name.toLowerCase()));
    }
  }

  bool isSearching = false;

  /// Prunes [adminUserTickets] and [adminUserData] to only contain the [search]ed term
  searchFor(String search) {
    if (search == "") {
      setState(() {
        adminUserData = allUserData;
        adminUserTickets = allUserTickets;
        isSearching = false;
      });
    } else  {
      adminUserData = [];
      adminUserTickets = [];
      isSearching = true;

      setState(() {
        for (var element in allUserTickets) {
          if (element.activatedBy.toLowerCase().contains(search.toLowerCase())) {
            adminUserTickets.add(element);
          } else if (element.purchasedBy.toLowerCase().contains(search.toLowerCase())) {
            adminUserTickets.add(element);
          } else if (element.giftCode.contains(search) && element.endDate == "") {
            adminUserTickets.add(element);
          }
        }
        for (var element in allUserData) {
          if (element.name.toLowerCase().contains(search.toLowerCase())) {
            adminUserData.add(element);
          } else if (element.email.toLowerCase().contains(search.toLowerCase())) {
            adminUserData.add(element);
          } else if (element.phone.contains(search)) {
            adminUserData.add(element);
          }
        }
      });
    }
  }

  bool u = false;
  bool t = false;

  Future<bool> ticketsComplete = firebaseGetAllTickets();
  Future<bool> usersComplete = firebaseGetAllUsers();

  bool showAllTickets = false;

  /// Add or subtracts expired tickets from [adminUserTickets] and [adminUserData]
  excludeIncludeExpired() {
    DateTime endDate = DateTime.now();
    if(!isSearching) {
      if (!showAllTickets) {
        adminUserTickets = [];
        for (var element in allUserTickets) {
          if (element.endDate == "") {
            adminUserTickets.add(element);
          } else {
            endDate = DateTime(int.parse(element.endDate.substring(0, 4)), int.parse(element.endDate.substring(4, 6)), int.parse(element.endDate.substring(6, 8)), 23, 59);
            if (endDate.compareTo(DateTime.now()) > 0) {
              adminUserTickets.add(element);
            }
          }

        }
      } else {
        adminUserTickets = allUserTickets;
      }
    }
    else if (!showAllTickets) {
      List<TicketData> temp = [];
      for (var element in adminUserTickets) {
        if (element.endDate == "") {
          temp.add(element);
        } else {
          endDate = DateTime(int.parse(element.endDate.substring(0, 4)),
              int.parse(element.endDate.substring(4, 6)),
              int.parse(element.endDate.substring(6, 8)), 23, 59);
          if (endDate.compareTo(DateTime.now()) > 0) {
            temp.add(element);
          }
        }
      }
      adminUserTickets = temp;
    }

  }

  static const List<String> sortByOptions = <String>['Dato', 'Alfabetisk'];
  String dropdownValue = sortByOptions.first;

  @override
  Widget build(BuildContext context) {

    // Ensure data is fetched from database
    ticketsComplete.whenComplete(() => setState(() {t = true;}));
    usersComplete.whenComplete(() => setState(() {u = true;}));
    excludeIncludeExpired();

    return Column(
      children: [
        Center(
          child: Padding(
            padding: const EdgeInsets.all(30.0),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceAround,
              children: [

                // Searchbar
                SizedBox(
                  width: 500,
                  child: TextField(
                    controller: _searchController,
                    decoration: const InputDecoration(
                      labelText: 'Søk',
                      border: OutlineInputBorder(),
                      filled: true,
                      fillColor: Colors.white,
                    ),
                    onChanged: (String value) {
                      //call function to update list that contains all items to be shown
                      searchFor(value);
                    },
                  ),
                ),

                // Choose what to sort by
                SizedBox(
                  //height: 50,
                  width: 100,
                  child: DropdownButton<String>(
                    value: dropdownValue,
                    onChanged: (String? value) {
                      setState(() {
                        dropdownValue = value!;
                        sortBy(value);
                      });
                    },
                    items: sortByOptions.map<DropdownMenuItem<String>>((String value) {
                      return DropdownMenuItem<String>(
                        value: value,
                        child: Text(value),
                      );
                    }).toList(),
                  ),
                ),

                // Add new user button
                SizedBox(
                  width: 150,
                  child: ElevatedButton(
                      style: const ButtonStyle (
                        backgroundColor: MaterialStatePropertyAll<Color>(Colors.green),
                      ),
                      onPressed: () {
                        //Open dialog window with a register form
                        _addNewUser(context);
                      },
                      child: const Text("Legg til bruker")
                  ),
                ),

                // Switch for including/excluding expired tickets
                SizedBox(
                  width: 200,
                  child: SwitchListTile(
                    title: const Text("Inkluder utgåtte billetter"),
                    onChanged: (bool b) {
                      setState(() {
                        showAllTickets = b;
                        excludeIncludeExpired();
                      });
                    },

                    value: showAllTickets,
                  ),
                ),
              ],
            ),
          ),
        ),
        // If data is fetched, show tickets and users
        if (u && t) AdminTicketTabs(),
      ],
    );

  }

  // Variables needed for new users
  DateTime currentDate = DateTime.now();
  String email = "";
  String firstName = "";
  String lastName = "";
  String phone = "";
  late Uint8List webProfilePic;

  bool imagePicked = false;
  var imagePath = 'assets/default.png';

  ///From: https://www.appsdeveloperblog.com/datepicker-example-in-flutter/
  Future<void> _selectDate(BuildContext context) async {
    final DateTime? pickedDate = await showDatePicker(
      context: context,
      initialDate: DateTime.now(),
      firstDate: DateTime(1920),
      lastDate: DateTime.now(),
    );
    if (pickedDate != null && pickedDate != currentDate) {
      setState(() {
        currentDate = pickedDate;
      });
    }
  }

  /// Pop-up window for adding a new user via the admin page
  _addNewUser(BuildContext context) {
    showDialog(
        context: context,
        builder: (context) {
          return StatefulBuilder(
              builder: (context, setState) {
                return AlertDialog(
                  title: const Text("Legg til ny bruker"),
                  content: SingleChildScrollView(
                    child: Form(
                      key: _formKey,
                      child: Column(
                        children: <Widget>[
                          // The commented out code underneath is for an image picker, which should be implemented in the future
                          /*if (!imagePicked)CircleAvatar(
                      maxRadius: 70,
                      backgroundImage: AssetImage(
                        imagePath,
                      ),
                    ),
                    if (imagePicked) CircleAvatar(
                      maxRadius: 70,
                      backgroundImage: MemoryImage(
                        webProfilePic,
                      ),
                    ),

                    SizedBox(height: 10,),

                    ElevatedButton(
                        onPressed: () async {
                          if (kIsWeb) {
                            XFile? image = await _picker.pickImage(source: ImageSource.gallery);

                            if (image != null){
                              var f = await image.readAsBytes();
                              webProfilePic = f;
                              setState(() {
                                imagePicked = true;
                              });
                            }
                          }
                        },
                        child: const Text("Legg til bilde")
                    ),*/

                          // First name
                          TextFormField(
                            decoration: const InputDecoration(
                              hintText: 'Fornavn',
                              icon: Icon(Icons.person),
                            ),
                            validator: (String? value) {
                              if (value == null || value.isEmpty) {
                                return 'Skriv inn fornavn';
                              }
                              firstName = value;
                              return null;
                            },
                          ),

                          // Last name
                          TextFormField(
                            decoration: const InputDecoration(
                              hintText: 'Etternavn',
                              icon: Icon(Icons.person),
                            ),
                            validator: (String? value) {
                              if (value == null || value.isEmpty) {
                                return 'Skriv inn etternavn';
                              }
                              lastName = value;
                              return null;
                            },
                          ),

                          // Email
                          TextFormField(
                            decoration: const InputDecoration(
                              hintText: 'Epost',
                              icon: Icon(Icons.mail),
                            ),
                            validator: (String? value) {
                              if (value == null || value.isEmpty) {
                                return 'Skriv inn epost';
                              }
                              email = value;
                              return null;
                            },
                          ),

                          // Phone number
                          TextFormField(
                            keyboardType: TextInputType.number,
                            decoration: const InputDecoration(
                              hintText: 'Mobilnummer',
                              icon: Icon(Icons.phone_android),
                            ),
                            validator: (String? value) {
                              if (value == null || value.isEmpty) {
                                return 'Skriv inn mobilnummer';
                              }
                              phone = value;
                              return null;
                            },
                          ),

                          // Date picker for birthday
                          Padding(
                            padding: const EdgeInsets.only(top: 8.0),
                            child: Row(
                              children: [
                                const Icon(Icons.date_range, color: Colors.grey,),
                                const Padding(
                                  padding: EdgeInsets.only(left: 15.0, right: 8.0),
                                  child: Text(
                                    "Fødselsdag:",
                                    style: TextStyle(
                                      fontSize: 17,
                                      color: Colors.blueGrey,
                                    ),),
                                ),
                                OutlinedButton(
                                    onPressed: () {
                                      setState((){
                                        _selectDate(context);
                                      });
                                    },
                                    child: Text(
                                      "${currentDate.day}/${currentDate.month}/${currentDate.year}",
                                      style: const TextStyle(
                                        fontSize: 17,
                                      ),
                                    )
                                ),
                              ],
                            ),
                          ),

                          // empty space
                          const SizedBox(height: 30,),

                          // Register button
                          ElevatedButton(
                            onPressed: () async {
                              if (_formKey.currentState!.validate()) {
                                //Add user to database
                                var uid = firebaseAddNonAuthenticatedUser(allUserData.length, "$firstName $lastName", email, phone,
                                    "${currentDate.day}/${currentDate.month}/${currentDate.year}");
                                setState(() {});
                                Navigator.of(context).pop();
                              }
                            },
                            child: const Text("Registrer"),
                          )
                        ],
                      ),
                    ),
                  ),
                );
              });
        }
    );
  }

}

//Page that can switch between two tabs (users and tickets)
//Both tabs contain a listview of all users/tickets as long as the search bar is empty
//if it is not, the list views contain only what matches or partially matches the searched letter/word/etc

///A tab page that keeps track of all users and all tickets
class AdminTicketTabs extends StatelessWidget {
  AdminTicketTabs({super.key});
  final ScrollController _scrollController = ScrollController();

  @override
  Widget build(BuildContext context) {
    return Expanded(

      child: DefaultTabController(
        length: 2,
        child: Scaffold(
          appBar: AppBar(
            bottom: const TabBar(
              tabs: [
                Tab(icon: Icon(Icons.person),),
                Tab(icon: Icon(Icons.confirmation_num)),
              ],
            ),
          ),
          body: TabBarView(
            children: [
              //Two lists, one for users and one for tickets
              Scrollbar(
                controller: _scrollController,
                scrollbarOrientation: ScrollbarOrientation.right,
                child: ListView.builder(
                  padding: const EdgeInsets.only(top: 0),
                  itemCount: adminUserData.length,
                  controller: _scrollController,
                  itemBuilder: (BuildContext context, int index) {return AdminTicketUsers(index); },
                ),
              ),

              Scrollbar(
                controller: _scrollController,
                scrollbarOrientation: ScrollbarOrientation.right,
                child: ListView.builder(
                  padding: const EdgeInsets.only(top: 0),
                  itemCount: adminUserTickets.length,
                  controller: _scrollController,
                  itemBuilder: (BuildContext context, int index) {return AdminTicketTickets(index); },
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

}

///A card with one users data, to be used in a list
class AdminTicketUsers extends StatefulWidget {
  const AdminTicketUsers(this.index, {super.key});
  final int index;

  @override
  State<AdminTicketUsers> createState() => _AdminTicketUsersState();
}

class _AdminTicketUsersState extends State<AdminTicketUsers> {
  final GlobalKey<FormState> _formKey = GlobalKey<FormState>();
  late TextEditingController _giftCodeController;

  @override
  void initState() {
    super.initState();
    _giftCodeController = TextEditingController();
  }

  @override
  void dispose() {
    _giftCodeController.dispose();
    super.dispose();
  }

  bool editMode = false;
  String newName = "";
  String newPhone = "";
  String newMail = "";
  String newBirthday = "";
  String giftCode = "";

  bool buyGiftCard = false;
  bool activateGiftCard = false;
  static const List<String> sortByOptions = <String>['Årskort', 'Voksenbillett', 'Barnebillett', 'Familiebillett'];
  String dropdownValue = sortByOptions.first;

  @override
  Widget build(BuildContext context) {
    // When not editing a user
    if (!editMode) {
      return Card(
        child: Padding(
          padding: const EdgeInsets.all(15.0),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Row(
                    children: [

                      // Name of user
                      Text(
                        adminUserData[widget.index].name,
                        style: const TextStyle(
                          fontSize: 25,
                          fontWeight: FontWeight.bold,
                        ),
                      ),

                      // Edit button
                      IconButton(
                        onPressed: () {
                          setState(() {
                            editMode = true;
                          });
                        },
                        icon: const Icon(Icons.edit),
                      )
                    ],
                  ),

                  const SizedBox(height: 30,),

                  // Phone number
                  Text(
                    adminUserData[widget.index].phone,
                    style: const TextStyle(
                        fontSize: 17
                    ),
                  ),

                  // Email
                  Text(
                    adminUserData[widget.index].email,
                    style: const TextStyle(
                        fontSize: 17
                    ),
                  ),

                  // Birthday
                  Text(
                    adminUserData[widget.index].birthday,
                    style: const TextStyle(
                        fontSize: 17
                    ),
                  ),

                  // Button for adding tickets
                  ElevatedButton(
                    onPressed: () {
                      _addTicket(context);
                    },
                    child: const Text("Legg til billett"),
                  ),

                ],
              ),
              // User image
              Image.network(adminUserData[widget.index].image, height: 200,),
            ],
          ),
        ),
      );

      // When editing a user
    } else {
      return Card(
        child: Padding(
          padding: const EdgeInsets.all(15.0),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SizedBox(
                width: (MediaQuery.of(context).size.width)/2,
                child: Form(
                  key: _formKey,
                  child: Column(
                    children: <Widget>[

                      // Name
                      TextFormField(
                        initialValue: adminUserData[widget.index].name,

                        style: const TextStyle(
                          fontSize: 25,
                          fontWeight: FontWeight.bold,
                        ),

                        validator: (String? value) {
                          if (value == null || value.isEmpty) {
                            return 'Må ha navn';
                          }
                          newName = value;
                          return null;
                        },
                      ),

                      // Empty space
                      const SizedBox(height: 30,),

                      // Phone number
                      TextFormField(
                        initialValue: adminUserData[widget.index].phone,
                        style: const TextStyle(
                            fontSize: 17
                        ),
                        validator: (String? value) {
                          if (value == null || value.isEmpty) {
                            return 'Må ha telefonnummer';
                          }
                          newPhone = value;
                          return null;
                        },
                      ),

                      // Email
                      TextFormField(
                        initialValue: adminUserData[widget.index].email,
                        style: const TextStyle(
                            fontSize: 17
                        ),
                        validator: (String? value) {
                          if (value == null || value.isEmpty) {
                            return 'Må ha epost';
                          }
                          newMail = value;
                          return null;
                        },
                      ),

                      // Warning
                      const Text("NB! Dette vil ikke oppdatere mailen som brukes til å logge inn"),

                      // Birthday
                      TextFormField(
                        decoration: const InputDecoration(
                            hintText: "dd/mm/åååå"
                        ),
                        initialValue: adminUserData[widget.index].birthday,
                        style: const TextStyle(
                            fontSize: 17
                        ),
                        validator: (String? value) {
                          if (value == null || value.isEmpty) {
                            return 'Må ha fødselsdagsdato';
                          }
                          newBirthday = value;
                          return null;
                        },
                      ),

                      // Is admin TODO: These should only be visible to admins
                      CheckboxListTile(
                        value: adminUserData[widget.index].isAdmin,
                        onChanged: (value) {
                          adminUserData[widget.index].isAdmin = value!;
                        },
                        title: const Text("Administrator"),
                      ),
                      //Is employee
                      CheckboxListTile(
                        value: adminUserData[widget.index].isEmployee,
                        onChanged: (value) {
                          adminUserData[widget.index].isEmployee = value!;
                        },
                        title: const Text("Ansatt"),
                      ),

                      //Empty space
                      const SizedBox(height: 30,),

                      // When user is done being edited, validate and update information
                      ElevatedButton(
                        onPressed: () {
                          if (_formKey.currentState!.validate()) {

                            if (newName != "") {adminUserData[widget.index].name = newName;}
                            if (newPhone != "") {adminUserData[widget.index].phone = newPhone;}
                            if (newMail != "") {adminUserData[widget.index].email = newMail;}
                            if (newBirthday != "") {adminUserData[widget.index].birthday = newBirthday;}

                            //Update data in database
                            Future<bool> update = firebaseUpdateUserInfo(adminUserData[widget.index]);

                            update.whenComplete(() => setState(() {
                              editMode = false;
                            }),
                            );
                          }

                        },
                        child: const Text("Ferdig"),
                      )
                    ],
                  ),
                ),
              ),
              Image.network(adminUserData[widget.index].image, height: 200,),
            ],
          ),
        ),
      );
    }

  }

  /// Pop-up for adding a ticket to a user
  /// Some code gathered from the accepted answer on this question:
  /// https://stackoverflow.com/questions/70529682/create-pop-up-dialog-in-flutter
  _addTicket(BuildContext context) {
    showDialog(
        context: context,
        builder: (context) {
          return StatefulBuilder(
              builder: (context, setState){
                return AlertDialog(
                  title: const Text("Legg til billett"),
                  content: SingleChildScrollView(
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.start,
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget> [

                        // Only show option for buying gift card, if they are not activating a gift card
                        if (!activateGiftCard) Container(
                            padding: const EdgeInsets.all(8.0),
                            child: SwitchListTile(
                              title: const Text("Kjøp gavekort"),
                              onChanged: (bool value) {
                                setState(() {
                                  buyGiftCard = value;
                                });
                              },
                              value: buyGiftCard,
                            )
                        ),

                        // Only show option for activating gift card, if they are not buying a gift card
                        if (!buyGiftCard) Container(
                            padding: const EdgeInsets.all(8.0),
                            child: SwitchListTile(
                              title: const Text("Løs inn gavekort"),
                              onChanged: (bool value) {
                                setState(() {
                                  activateGiftCard = value;
                                });
                              },
                              value: activateGiftCard,
                            )
                        ),

                        // If they are buying a gift card, show a gidt card code
                        if (buyGiftCard) Container(
                          padding: const EdgeInsets.all(4.0),
                          decoration: BoxDecoration(
                              border: Border.all()
                          ),
                          child: const Text("This is where gift card codes will be"),
                        ),

                        // If they are not buying a gift card, let them choose ticket type
                        if (!activateGiftCard) DropdownButton<String>(
                            value: dropdownValue,
                            onChanged: (String? value) {
                              setState(() {
                                dropdownValue = value!;
                              });
                            },
                            items: sortByOptions.map<DropdownMenuItem<String>>((String value) {
                              return DropdownMenuItem<String>(
                                value: value,
                                child: Text(value),
                              );
                            }).toList()
                        ),

                        // When activating a gift card, show a place to input the code
                        if (activateGiftCard) Container(
                          padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
                          child: TextField(
                            controller: _giftCodeController,
                            decoration: const InputDecoration(
                                border: OutlineInputBorder(),
                                labelText: "Gavekortkode"
                            ),
                            onChanged: (value) {
                              giftCode = value;
                            },
                          ),
                        ),

                        // Button for validating ticket, and uploading correctly
                        ElevatedButton(
                          onPressed: () {
                            //Establish today's date as a string
                            String date = DateTime.now().year.toString() + ((DateTime.now().month.toString().length == 1)? "0": "") + DateTime.now().month.toString() +
                                ((DateTime.now().day.toString().length == 1)? "0": "") + DateTime.now().day.toString();

                            // Establish end date of ticket, this varies based on ticket type, and whether or not it is a gift card
                            String endDate = "";
                            if (buyGiftCard) {
                              endDate = "";
                            } else if (dropdownValue == "Årskort" || activateGiftCard) {
                              var aYearTo = DateTime.now().add(const Duration(days: 365));
                              endDate = aYearTo.year.toString() + ((DateTime.now().month.toString().length == 1)? "0": "") +
                                  aYearTo.month.toString() + ((DateTime.now().day.toString().length == 1)? "0": "") + aYearTo.day.toString();

                            } else {
                              endDate = date;
                            }

                            // Ticket data
                            TicketData newTicket = TicketData(
                                activatedBy: (buyGiftCard)? "" : adminUserData[widget.index].name,
                                activationDate: (buyGiftCard)? "" : date,
                                endDate: endDate,
                                purchaseDate: date,
                                purchasedBy: adminUserData[widget.index].name,
                                giftCode: (buyGiftCard)? "Gift code goes here" : "",
                                type: dropdownValue
                            );

                            // When buying a gift card, the ticket is simply pushed to the database
                            if (buyGiftCard) {
                              Future<bool> uploaded = firebaseAddTicketToDatabase(newTicket, adminUserData[widget.index]);
                              uploaded.whenComplete(() => Navigator.of(context).pop());

                              // When activating a gift card, find the correct ticket and add to user
                            } else if (activateGiftCard) {

                              var ticket = null;
                              //Find the correct ticket
                              for (var element in allUserTickets) {
                                //Gift codes have to match, and also making sure that this tickets is not already activated
                                if (element.giftCode == giftCode && element.endDate == "") {
                                  ticket = element;
                                  break;
                                }
                              }
                              if (ticket != null) {
                                //Update information (activation date, activated by, end date)
                                ticket.endDate = endDate;
                                ticket.activationDate = date;
                                ticket.activatedBy = adminUserData[widget.index].name;

                                firebaseUpdateTicket(ticket);

                                //Ticket added to user
                                adminUserData[widget.index].tickets = adminUserData[widget.index].tickets + "," + ticket.ticketID;

                                firebaseUpdateUserInfo(adminUserData[widget.index]);
                              }
                              Navigator.of(context).pop();

                              // Otherwise upload ticket to database and add to user
                            } else {
                              Future<bool> uploaded = firebaseAddTicketToDatabase(newTicket, adminUserData[widget.index], adminUserData[widget.index].userID);
                              uploaded.whenComplete(() => Navigator.of(context).pop());
                            }

                          },
                          child: const Text("Legg til"),
                        )
                      ],
                    ),
                  ),
                );
              }
          );
        }
    );
  }

}

///A card with one tickets data, to be used in a list
class AdminTicketTickets extends StatelessWidget {
  AdminTicketTickets(this.index, {super.key});
  final int index;

  DateTime endDate = DateTime.now();
  int expiration = 0;

  /// Checks if the ticket is expired or not
  checkExpiration() {
    if (adminUserTickets[index].endDate != "") {
      endDate = DateTime(int.parse(adminUserTickets[index].endDate.substring(0, 4)),
          int.parse(adminUserTickets[index].endDate.substring(4, 6)),
          int.parse(adminUserTickets[index].endDate.substring(6, 8)), 23, 59);
      if (endDate.compareTo(DateTime.now()) > 0) {
        expiration = 1;
      } else {
        expiration = 2;
      }
    }
  }


  @override
  Widget build(BuildContext context) {
    checkExpiration();
    return Card(
      child: Column(
        children: [

          // Non-activated tickets
          if (expiration == 0) Container(
              alignment: Alignment.centerLeft,
              padding: const EdgeInsets.all(8.0),
              child: Text(
                "${adminUserTickets[index].type} (ikke aktivert)",
                style: const TextStyle(
                  fontSize: 25,
                ),
              )
          ),

          // Active tickets
          if (expiration == 1) Container(
              alignment: Alignment.centerLeft,
              padding: const EdgeInsets.all(8.0),
              child: Text(
                adminUserTickets[index].type,
                style: const TextStyle(
                    fontSize: 25,
                    color: CupertinoColors.activeGreen
                ),
              )
          ),

          // Exoired tickets
          if (expiration == 2) Container(
              alignment: Alignment.centerLeft,
              padding: const EdgeInsets.all(8.0),
              child: Text(
                "${adminUserTickets[index].type} (utgått)",
                style: const TextStyle(
                    fontSize: 25,
                    color: Colors.red
                ),
              )
          ),

          // Yearly memberships are able to be reactivated
          if (adminUserTickets[index].type == "Årskort" && expiration == 2) Container(
            alignment: Alignment.centerLeft,
            padding: const EdgeInsets.all(8.0),
            child: IconButton(
              onPressed: () {
                //figure out today's date, that is new purchase date
                adminUserTickets[index].purchaseDate = DateTime.now().year.toString() + ((DateTime.now().month.toString().length == 1) ? "0" : "") +
                    DateTime.now().month.toString() + ((DateTime.now().day.toString().length == 1) ? "0" : "") + DateTime.now().day.toString();
                //today's date plus a year is new expiration date
                adminUserTickets[index].endDate = DateTime.now().add(const Duration(days: 365)).year.toString() +
                    ((DateTime.now().add(const Duration(days: 365)).month.toString().length == 1) ? "0" : "") +
                    DateTime.now().add(const Duration(days: 365)).month.toString() + ((DateTime.now().add(const Duration(days: 365)).day.toString().length == 1) ? "0" : "") +
                    DateTime.now().add(const Duration(days: 365)).day.toString();
                //send ticket to firebase
                firebaseUpdateTicket(adminUserTickets[index]);
              },
              icon: const Icon(Icons.repeat),
            ),
          ),

          // Information about ticket
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: Row(
              children: [
                Text(
                  "Kjøpt dato: ${adminUserTickets[index].purchaseDate.substring(6)} / ${adminUserTickets[index].purchaseDate.substring(4, 6)} / ${adminUserTickets[index].purchaseDate.substring(0, 4)}",
                  style: const TextStyle(
                    fontSize: 17,
                  ),
                ),
                const SizedBox(width: 30,),
                Text(
                  "Aktivert dato: ${(adminUserTickets[index].activationDate.isNotEmpty) ?
                  "${adminUserTickets[index].activationDate.substring(6)} / ${adminUserTickets[index].activationDate.substring(4, 6)} / ${adminUserTickets[index].activationDate.substring(0, 4)}"
                      : "Ikke aktivert" }",
                  style: const TextStyle(
                    fontSize: 17,
                  ),
                ),
                const SizedBox(width: 30,),
                Text(
                  "Utløpsdato: ${(adminUserTickets[index].endDate.isNotEmpty) ?
                  "${adminUserTickets[index].endDate.substring(6)} / ${adminUserTickets[index].endDate.substring(4, 6)} / ${adminUserTickets[index].endDate.substring(0, 4)}"
                      : "Ikke aktivert" }",
                  style: const TextStyle(
                    fontSize: 17,
                  ),
                ),
              ],
            ),
          ),
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: Row(
              children: [
                Text("Kjøpt av: ${adminUserTickets[index].purchasedBy}",
                  style: const TextStyle(
                    fontSize: 17,
                  ),
                ),
                const SizedBox(width: 30,),

                if (adminUserTickets[index].giftCode != "" && adminUserTickets[index].activatedBy == "") Text(
                  "Gavekortkode: ${adminUserTickets[index].giftCode}",
                  style: const TextStyle(
                      fontSize: 17,
                      fontWeight: FontWeight.bold
                  ),
                ),

                if (adminUserTickets[index].giftCode ==  "" || adminUserTickets[index].activatedBy != "") Text(
                  "Aktivert av: ${adminUserTickets[index].activatedBy}",
                  style: const TextStyle(
                    fontSize: 17,
                  ),
                ),
              ],
            ),
          )
        ],
      ),
    );
  }

}
