import 'package:billettapp_for_vitensenteret/admin_blog_page.dart';
import 'package:billettapp_for_vitensenteret/resources/structs.dart';
import 'package:expandable/expandable.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:loading_animation_widget/loading_animation_widget.dart';
import 'colors.dart';
import 'firebase_functions.dart';

/// Page for the blog
class BlogPage extends StatefulWidget{
  const BlogPage({super.key});

  @override
  State<StatefulWidget> createState() => _BlogPageState();
}

class _BlogPageState extends State<BlogPage>{

  //Firebase variables
  bool fetchedData = false;
  Future<void> fetchedDataCheck = firebaseGetBlog();
  bool fetchedDataUser = false;
  Future<void> fetchedDataCheckUser = firebaseGetUser();


  @override
  Widget build(BuildContext context) {
    List<Widget> list = List.generate(allBlogs.length, (index) => _individualBlogPost(index), growable: true);

    // Ensure data from database is fetched for blogs and user
    if(!fetchedData){
      fetchedDataCheck.whenComplete(() => setState((){
        fetchedData = true;
        }));
    }
    if(!fetchedDataUser){
      fetchedDataCheckUser.whenComplete(() => setState((){
        fetchedDataUser = true;
      }));
    }

    // Only show the admin page to an admin using the web platform
    if (fetchedData && currentUser.isAdmin && kIsWeb){
      return MaterialApp(
        home: Scaffold(

          // Appbar with button to navigate to admin page
          appBar: AppBar(
              actions: <Widget>[
                IconButton(
                  icon: const Icon(Icons.edit),
                  tooltip: 'Admin side for blog',
                  onPressed: () => Navigator.push(
                      context,
                  MaterialPageRoute(builder: (context) => const AdminPageForTheBlog()),
                  ),
                ),
            ],
            title: const Text('Trykk her for Admin siden for Blog'),
            titleSpacing: 00.0,
            centerTitle: true,
            toolbarHeight: 60.2,
            toolbarOpacity: 0.8,
            elevation: 0.00,
            backgroundColor: morkBlaa,),
          body: Center(
            child:
            Container(
              color: blaa,
              child: ListView(
                padding: const EdgeInsets.all(12),
                children: list.reversed.toList(),
              ),
            ),
          ),
        ),
      );

      // If all data is fetched, show blogs
    } else if (fetchedData) {
      return Center(
        child:
        Container(
          color: blaa,
          child: ListView(
            padding: const EdgeInsets.all(12),
            children: list.reversed.toList(),
          ),
        ),
      );
    }

    // If data is not ready yet, show a loading screen
    else {
      return SizedBox.expand( //Loading screen until data is fetched from firebase
        child: Container(
          color: blaa,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              LoadingAnimationWidget.staggeredDotsWave(
                size: 100,
                color: morkBlaa,
              ),
            ],
          ),
        ),
      );
    }
  }
}


/// Widget for each individual blog post
Widget _individualBlogPost(int index){
  return Card(
    clipBehavior: Clip.antiAlias,
    child: Padding(
      padding: const EdgeInsets.all(12),
      child: ExpandablePanel(

        // Title and publication date
        header: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Text(allBlogs[index].title, style: const TextStyle(fontWeight: FontWeight.bold),),
            Text(allBlogs[index].publicationDate, style: const TextStyle(color: Colors.grey),)
          ],
        ),

        // Blog content in collapsed form
        collapsed: Text(allBlogs[index].content, softWrap: true, maxLines: 1, overflow: TextOverflow.ellipsis,),

        // Author, image and blog content when a blog post is clicked on
        expanded: Column(
          children: [
            Text(allBlogs[index].author),
            if(allBlogs[index].image.isNotEmpty)
              Image.network(allBlogs[index].image),
            const SizedBox(height: 50),
            Text(allBlogs[index].content, softWrap: true),
          ]
        )
      ),
    ),
  );
}

/// Redirects to the admin page for the blog
class AdminPageForTheBlog extends StatelessWidget {
  const AdminPageForTheBlog({super.key});

  @override
  Widget build(BuildContext context) {
    //var appState = context.watch<Notifier>();
    return const MaterialApp(
      title: 'Flutter BlogPage',
      debugShowCheckedModeBanner: false,
      home: AdminBlogPage(),
    );
  }
}
