import 'dart:developer';

import 'package:billettapp_for_vitensenteret/firebase_options.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:billettapp_for_vitensenteret/blog.dart';
import 'package:flutter_stripe/flutter_stripe.dart';
import 'package:billettapp_for_vitensenteret/home_games.dart';
import 'package:billettapp_for_vitensenteret/settings.dart';
import 'authentication.dart';

void main() async{

  // Initialize Firebase and Stripe
  WidgetsFlutterBinding.ensureInitialized();
  try {
    await Firebase.initializeApp(options: DefaultFirebaseOptions.currentPlatform);
  }
  catch(err) {
    log("Error initializing app: $err");
  }
  if (!kIsWeb) {
    Stripe.publishableKey = "pk_test_51MvcQJDQJVx39bXEUIoALvvnSpeaz4r1CrvkH3oJ6r42ndYtYShrj6ysUh1WCeLEDnkIDj7wrhbE22C8sJcRJ6GH00I1nEYaYy";
    Stripe.instance.applySettings();
  }

  // Run the app
  runApp(const MyApp());
}

/// Sets ip the app and navigation
class MyApp extends StatelessWidget {
  const MyApp({super.key});

  @override
  Widget build(BuildContext context) {
    return MaterialApp (
        title: 'Vitensenteret App',
        theme: ThemeData(
            primarySwatch: const MaterialColor(0xFF96d5d2, <int, Color>{
              50: Color(0xFF2c6263),
              100: Color(0xFF2c6263),
              200: Color(0xFF2c6263),
              300: Color(0xFF2c6263),
              400: Color(0xFF2c6263),
              500: Color(0xFF96d5d2),
              600: Color(0xFFa8c67e),
              700: Color(0xFFe9d74f),
              800: Color(0xFFe4c1bb),
              900: Color(0xFFc8513c),
            }),
        ),
        home: const PageSelection(),
    );
  }
}

// Index in navigation
var selectedIndex = 0;

/// Navigation bar, both on the bottom and on the side
class PageSelection extends StatefulWidget {
  const PageSelection({super.key});

  @override
  State<PageSelection> createState() => _PageSelectionState();
}

class _PageSelectionState extends State<PageSelection> {

  // Change the selected index when user navigates to a new page
  void _onItemTapped(int i) {
    setState(() {
      selectedIndex = i;
    });
  }

  @override
  Widget build(BuildContext context) {
    Widget page;
    switch (selectedIndex) {
      case 0:
        page = BlogPage();
        break;
      case 1:
        page = GamePage(); //This page contains the selection of games available
        break;
      case 2:
        page = AuthenticationPage(); // This page contains log in and register page, will redirect to ticket page if a user is authenticated
        break;
      case 3:
        page = ProfilePage(); // This page allows users to change user data, log out, or delete user
        break;
      default:
        throw UnimplementedError('Page for index $selectedIndex not implemented');
    }

    return LayoutBuilder(builder: (context, constraints) {
      // If the screen width is greater than a certain width, the navigation is on the side
      if (constraints.maxWidth >= 500) {
        //Todo: Maybe code something more dynamic here
        return Scaffold(
          body: Row(
            children: [
              SafeArea(
                child: NavigationRail(
                  extended: constraints.maxWidth >= 800,
                  destinations: const [
                    NavigationRailDestination(
                        icon: ImageIcon(AssetImage("assets/images/green-logo-vertical.png")),
                        label: Text('Vitensenteret')),
                    NavigationRailDestination(
                      icon: Icon(Icons.gamepad),
                      label: Text('Spill')),
                    NavigationRailDestination(
                      icon: Icon(Icons.confirmation_num),
                      label: Text('Billett')),
                    NavigationRailDestination(
                        icon: Icon(Icons.settings),
                        label: Text('Innstillinger'))
                  ],
                  selectedIndex: selectedIndex,
                  onDestinationSelected: (value) {
                    setState(() {
                      selectedIndex = value;
                    });
                  },
                )
              ),
              Expanded(
                  child: Container(
                    color: Theme.of(context).colorScheme.primaryContainer,
                    child: page,
                  )
              )
            ],
          ),
        );
      }

      // Navigation bar on the bottom of the screen is the default
      return Scaffold(
        body: Center(
          child: Container(
            color: Theme.of(context).colorScheme.primaryContainer,
            child: page,
          ),
        ),
        bottomNavigationBar: BottomNavigationBar(
          unselectedIconTheme: const IconThemeData(color: Colors.blueGrey),
          items: const <BottomNavigationBarItem>[
            BottomNavigationBarItem(
                icon: ImageIcon(AssetImage("assets/images/green-logo-vertical.png")),
                label: 'Vitensenteret'),
            BottomNavigationBarItem(
                icon: Icon(Icons.gamepad),
                label: 'Spill'),
            BottomNavigationBarItem(
                icon: Icon(Icons.confirmation_num),
                label: 'Billett'),
            BottomNavigationBarItem(
                icon: Icon(Icons.settings),
                label: 'Innstillinger'),
          ],
          currentIndex: selectedIndex,
          selectedItemColor: Theme.of(context).colorScheme.primary,
          onTap: _onItemTapped,
        ),
      );
    });
  }
}



