import 'package:billettapp_for_vitensenteret/firebase_functions.dart';
import 'package:billettapp_for_vitensenteret/resources/structs.dart';
import 'package:flutter/material.dart';
import 'package:billettapp_for_vitensenteret/home_games.dart';
import 'package:animated_text_kit/animated_text_kit.dart';

import '../colors.dart';

void main() => runApp(const ProgrammingGame());

/// Contains programming game widget
class ProgrammingGame extends StatelessWidget {
  const ProgrammingGame({super.key});

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          leading: BackButton(
            onPressed: () => Navigator.push(
              context,
              MaterialPageRoute(builder: (context) => const GameMenuPage()),
            ),
          ),
          title: const Text('Viten Kode'),
          titleSpacing: 00.0,
          centerTitle: true,
          toolbarHeight: 60.2,
          toolbarOpacity: 0.8,
          shape: const RoundedRectangleBorder(
            borderRadius: BorderRadius.only(
                bottomRight: Radius.circular(25),
                bottomLeft: Radius.circular(25)),
          ),
          elevation: 0.00,
          backgroundColor: morkBlaa,),
        body: const ProgrammingGameWidget(),
      ),
    );
  }
}

/// Main game loop of programming game
class ProgrammingGameWidget extends StatefulWidget {
  const ProgrammingGameWidget({super.key});

  @override
  State<ProgrammingGameWidget> createState() => _MyStatefulWidgetState();
}

class _MyStatefulWidgetState extends State<ProgrammingGameWidget> {
  // Rules
  final String gamedescription = "1. Målet er å dra utsagnende slik at de kommer i rett rekkefølge.\n"
      "2. Når du er fornøyd trykker du på sjekk, då får du resultat.\n\n"
      "                                            Lykke til!";
  // Firebase variables
  bool fetchedData = false;
  Future<void> fetchedDataCheck = firebaseGetVitenKode();

  // Game logic variables
  var taskId = 0;
  var task = "";
  List<String> solution = [];
  List<String> currentList = [];
  bool win = false;
  bool check = false;
  late var _cells = [];


  @override
  Widget build(BuildContext context) {
    // Check if data has been fetched from database
    if(!fetchedData){
      // Setting all variables, some are fetched, some are manually set
      fetchedDataCheck.whenComplete(() => setState((){
        taskId = getRandomId();
        fetchedData = true;
        task = getTask(taskId);
        solution = getOptions(taskId);
        currentList = getShuffledSolution(taskId);
        win = false;
        check = false;
        _cells = generateList(currentList);
      }));
    }
    // Start game when data has been fetched
    if(fetchedData) {
      return Scaffold(
      body: SafeArea(
        child: Column(
          children: <Widget>[
            // Display a task
            Container(
              padding: const EdgeInsets.all(20),
              margin: const EdgeInsets.all(10),
              height: 100,
              color: blaa,
              child: Center(child: Text(getTask(taskId))),
            ),
            // Reorderable list containing options (main part of game)
            Flexible(
              child: ListView(
                shrinkWrap: true,
                children: [
                  Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      SizedBox(
                          height: 300,
                          child: _reorder),
                    ],
                  )
                ],
              ),
            ),
            // Bottom menu objects
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                // Display rules
                TextButton(
                  onPressed: () => showDialog<String>(
                    context: context,
                    builder: (BuildContext context) => AlertDialog(
                      title: const Text('Hvordan spille Viten Kode:'),
                      content: Text(gamedescription),
                      actions: const <Widget>[
                      ],
                    ),
                  ),
                  style: TextButton.styleFrom(
                    foregroundColor: morkBlaa,
                  ),
                  child: const Text('Regler'),
                ),
                // Skip current task
                TextButton(
                  style: ButtonStyle(
                    foregroundColor: MaterialStateProperty.all<Color>(morkBlaa),
                  ),
                  onPressed: () {
                    setState(() {
                    check = false;
                    win = false;
                    solution = getOptions(taskId);
                    currentList = getShuffledSolution(taskId);
                    _cells = generateList(currentList);
                  });
                  },
                  child: const Text('Hopp Over'),
                ),
                // Check if list has been ordered correctly
                TextButton(
                  style: ButtonStyle(
                    foregroundColor: MaterialStateProperty.all<Color>(morkBlaa),
                  ),
                  onPressed: () {
                    setState(() {
                      check = true;
                      if (checkWin(solution, currentList)) {
                        check = false;
                        win = true;
                      }
                    });
                  },
                  child: const Text('Sjekk'),
                )
              ],
            ),
            // When player wins
            if(check)
              Container(padding: const EdgeInsets.all(20),
                margin: const EdgeInsets.all(10),
                height: 100,
                color: rod,
                child: const Center(child: Text("Feil, prøv videre, eller hopp over oppgaven",
                    style: TextStyle(color: Colors.white  ))),),
            // When player wins
            if(win)
              Column(
                children: [
                  Container(
                    height: 50.0,
                    width: 200.0,
                    decoration: const BoxDecoration(
                        color: gronn,
                        boxShadow: [
                          BoxShadow(
                              color: Colors.grey ,
                              blurRadius: 2.0,
                              offset: Offset(2.0,2.0)
                          )
                        ]
                    ),
                    child: Center(
                        child: _victory()
                    ),
                  ),
                  TextButton(
                    style: TextButton.styleFrom(
                      foregroundColor: morkBlaa,
                    ),
                    onPressed: () {
                      setState(() {
                        solution = getOptions(taskId);
                        currentList = getShuffledSolution(taskId);
                        _cells = generateList(currentList);
                        win = false;
                      });
                    },
                    child: const Text("Neste"),
                  ),
                ],
              ),
          ],
        ),
      ),
    );
    } else {return Container();} // Empty page while data is being fetched todo: Loading screen
  }

  /// Reorderable listview widget, solution was inspired by:
  /// https://stackoverflow.com/questions/61628229/reorderablelistview-inside-singlechildscrollview-column
  Widget get _reorder =>
      ReorderableListView.builder(
        physics: const ClampingScrollPhysics(),
        shrinkWrap: true,
        itemBuilder: (BuildContext context, int index) => _cells[index],
        itemCount: _cells.length,
        onReorder: (int oldIndex, int newIndex) {
          setState(() {
            // This happens when a user shuffles or moves items, it inserts
            // the moved item to its new place,
            // and deletes it from where it used to be
            if (oldIndex < newIndex) {
              newIndex -= 1;
            }
            final ListTile item = _cells.removeAt(oldIndex);
            final String option = currentList.removeAt(oldIndex);
            _cells.insert(newIndex, item);
            currentList.insert(newIndex, option);
          });
        },
      );
}


/// Gets task description from task with [taskId].
/// Returns a string, which is the task description
String getTask(int taskId) {
  return allTaskData[taskId].task;
}


/// Get list of string descriptions of all options a task with [taskId]
/// Returns a list of strings of options
List<String> getOptions(int taskId){
  return[allTaskData[taskId].one, allTaskData[taskId].two, allTaskData[taskId].three, allTaskData[taskId].four, allTaskData[taskId].five];
}


/// Returns a random int from zero to the total amount of tasks
/// Currently only returns 0 for testing purposes
int getRandomId(){
  if(allTaskData.isEmpty){
    return 0;
  }
  return 0;
}


/// Takes the solution of [taskId] and shuffles the order of options
/// Returns a reordered list of strings of options
List<String> getShuffledSolution(int taskId){
  List<String> solution = getOptions(taskId);
  solution.shuffle(); //shuffle solution here
  return solution;
}


/// Takes a string list containing all options [currentList] and converts it to a lListTile list
/// Returns the a ListTile list
List<ListTile> generateList(List<String> currentList){
  var cells = List.generate(5, (index) {
    return ListTile(
      key: Key('$index'),
      title: Text(
        currentList[index], //Here is how we grab the option description
      ),
    );
  });

  return cells;
}


/// Takes two list of strings, one is the solution: [solution], the other is current order: [currentList]
/// If the lists match, returns true, otherwise false
bool checkWin(List<String> solution, List<String> currentList){
  for(int i = 0; i < solution.length; i++){
    if(solution[i] != currentList[i]){
      return false;
    }
  }
  return true;
}


/// Widget for the victory screen text animation
/// From public flutter: https://pub.dev/packages/animated_text_kit
Widget _victory(){
  return DefaultTextStyle(
    style: const TextStyle(
      fontSize: 25.0,
    ),
    child: AnimatedTextKit(
      animatedTexts: [
        WavyAnimatedText("Gratulerer!",
            speed: const Duration(milliseconds: 200)),
      ],
      isRepeatingAnimation: true,
      repeatForever: true,
    ),
  );
}
