/// Contains functions that can be used multiple times
import 'dart:math';

/// Validates that a string [mail] is an email address
bool validateEmail(String mail) {
  if (!mail.contains(RegExp(r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9]+\.[a-zA-Z]+"))) {
    return false;
  }

  return true;
}

/// Validates that a string [num] could be a potential phone-number
bool validatePhone(String num) {
  if (!num.contains(RegExp(r"^[0-9+]*$"))) {
    return false;
  }
  return true;
}

/// Validates that a string [pass] follows password conventions
int validatePassword(String pass) {
  // Invalid length
  if (pass.length < 8) {
    return 1;
  }
  // Does not contain a letter
  if (!pass.contains(RegExp(r"^(?=.*[a-zA-Z])"))) {
    return 2;
  }
  // Does not contain a number
  if (!pass.contains(RegExp(r"^(?=.*?[0-9])"))) {
    return 3;
  }
  // Valid
  return 0;
}

/// Generate a random string with upper- and lowercase letters, and numbers
/// Code from: https://stackoverflow.com/questions/61919395/how-to-generate-random-string-in-dart
const _chars = 'AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz1234567890';
Random _rnd = Random();

String getRandomString(int length) => String.fromCharCodes(Iterable.generate(
    length, (_) => _chars.codeUnitAt(_rnd.nextInt(_chars.length))));

