import 'package:billettapp_for_vitensenteret/colors.dart';
import 'package:billettapp_for_vitensenteret/payment.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:billettapp_for_vitensenteret/resources/structs.dart';
import 'firebase_functions.dart';

/// The ticket page, with profile picture, name and a widget with either tickets,
/// info about a ticket, or ticket purchase information
class TicketsPage extends StatefulWidget {
  const TicketsPage({super.key});

  @override
  State<TicketsPage> createState() => _TicketsPageState();
}

class _TicketsPageState extends State<TicketsPage>
  with TickerProviderStateMixin {
  late AnimationController controller;

  bool showTickets = true;
  bool ticketsLoaded = false;
  var buttonText = '+';
  double sizeOfTickets = 240;

  Future<void> getTickets = firebaseGetTickets();

  @override
  void initState() {
    controller = AnimationController(
      /// [AnimationController]s can be created with `vsync: this` because of
      /// [TickerProviderStateMixin].
      vsync: this,
      duration: const Duration(minutes: 10),
    )..addListener(() {
      setState(() {});
    });
    controller.forward();
    super.initState();
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {

    // Variables that have o be initialized inside a widget
    var profilePic = FirebaseAuth.instance.currentUser!.photoURL;
    var profileName = FirebaseAuth.instance.currentUser!.displayName;
    var screenHeight = MediaQuery.of(context).size.height;

    // Ensure all tickets are fetched from database
    if(!ticketsLoaded) {
      getTickets.whenComplete(() => setState(() {ticketsLoaded = true;}));
    }

    return Padding(
          padding: EdgeInsets.only(left: 8.0, right: 8.0, top: screenHeight/15),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.stretch,
            mainAxisSize: MainAxisSize.max,
            children: [
              SizedBox(
                height: screenHeight/3,
                child: Column(
                    mainAxisSize: MainAxisSize.max,
                    children: [

                      // Show profile picture if it exists
                      if (profilePic != null) CircleAvatar(
                        maxRadius: screenHeight/10,
                        backgroundImage: NetworkImage(
                          profilePic,
                        ),
                      ),

                      // Empty space
                      SizedBox(height: screenHeight/40,),

                      // Show profile name if it exists
                      if (profileName != null) SizedBox(
                        height: screenHeight/15,
                        child: FittedBox(
                          fit: BoxFit.contain,
                          child: Text(
                            profileName,
                            style: const TextStyle (
                              fontSize: 20,
                              color: morkBlaa,
                            ),
                          ),
                        ),
                      ),
                    ]
                ),
              ),

              Expanded(
                child: Align(
                  alignment: Alignment.bottomCenter,
                  child: Card(
                    shape: const RoundedRectangleBorder(
                      borderRadius: BorderRadius.all(Radius.circular(10)),
                    ),
                    //Has to be changed to scrollable list
                    child: Stack(
                      children: [

                        // Page with all user tickets
                        _Tickets(),

                        //Button that redirects to payment page
                        Stack(
                          children: [
                            Container(
                              alignment: Alignment.bottomRight,
                              padding: const EdgeInsets.all(8.0),
                              child: SizedBox(
                                height: 75,
                                width: 75,
                                child: FittedBox(
                                  child: FloatingActionButton(
                                    onPressed: () {
                                      Navigator.push(context, MaterialPageRoute(builder: (context) => const PaymentPage()));
                                    },
                                    backgroundColor: rosa,

                                    child: const Text(
                                      "+",
                                      style: TextStyle(
                                        fontSize: 35,
                                        color: Colors.white,
                                      ),
                                    ),
                                  ),
                                ),
                              ),
                            )
                          ],
                        ),
                      ],
                    ),
                  ),
                ),
              ),
            ],
          ),
    );

  }
}

///Overview over all tickets, or all information about one ticket
class _Tickets extends StatefulWidget {
  @override
  State<_Tickets> createState() => _TicketsState();
}

class _TicketsState extends State<_Tickets> {
  final ScrollController _firstController = ScrollController();

  @override
  Widget build(BuildContext context) {
    return Scrollbar(
      controller: _firstController,
      scrollbarOrientation: ScrollbarOrientation.right,
        child: ListView.builder(
          padding: const EdgeInsets.only(top: 0), //remove default listview padding
          itemCount: userTickets.length,
          controller: _firstController,
          itemBuilder: (BuildContext context, int index) { return _Ticket(index: index,); },

        )
    );
  }
}

/// One ticket that the users owns
class _Ticket extends StatefulWidget{
  final int index;

  const _Ticket({Key? key, required this.index}) : super(key: key);

  @override
  State<_Ticket> createState() => _TicketState();
}

class _TicketState extends State<_Ticket>
  with TickerProviderStateMixin{
  late AnimationController controller;
  
  DateTime startDate = DateTime.now();
  DateTime endDate = DateTime.now();

  @override
  void initState() {
    super.initState();
    TicketData ticket = userTickets[widget.index];
    //TODO: If DateFormat can be used to convert, user that
    // Find start date and end date
    startDate = DateTime(int.parse(ticket.activationDate.substring(0, 4)), int.parse(ticket.activationDate.substring(4, 6)), int.parse(ticket.activationDate.substring(6, 8)), 0, 1);
    endDate = DateTime(int.parse(ticket.endDate.substring(0, 4)), int.parse(ticket.endDate.substring(4, 6)), int.parse(ticket.endDate.substring(6, 8)), 23, 59);

    // Find how much time is left on the tickets
    int dur = endDate.difference(startDate).inHours + 1;
    double diff = endDate.difference(DateTime.now()).inHours.toDouble();

    controller = AnimationController(
      /// [AnimationController]s can be created with `vsync: this` because of
      /// [TickerProviderStateMixin].
      vsync: this,
      reverseDuration: Duration(hours: dur),
      value: diff/dur,
    )..addListener(() {
      setState(() {});
    });
    controller.reverse();
    super.initState();
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Card(
      color: Theme.of(context).primaryColor,
      child: Column(
        children: [

          // Type of ticket
          Padding(
            padding: const EdgeInsets.only(top: 8.0, left: 10.0, right: 10.0),
            child: Align(
              alignment: Alignment.topLeft,
              child: Text(
                userTickets[widget.index].type,
                style: const TextStyle(
                    fontSize: 35
                ),
              ),
            ),
          ),

          // When does ticket expire
          Padding(
            padding: const EdgeInsets.only(top: 4.0, left: 10.0, right: 10.0),
            child: Align(
              alignment: Alignment.topLeft,
              child: Text(
                'Billett utløper: ${endDate.day}-${endDate.month}-${endDate.year}',
                style: const TextStyle(
                    fontSize: 15,
                    color: Colors.grey
                ),
              ),
            ),
          ),

          // Progress indicator for how long the ticket is valid for
          Padding(
            padding: const EdgeInsets.only(top: 12.0, left: 10.0, right: 10.0, bottom: 32.0),
            child: LinearProgressIndicator(
              value: controller.value,
              minHeight: 30,
              //https://pub.dev/packages/animated_background for animated background??
              backgroundColor: Colors.grey,
              color: rod,
            ),
          ),
        ]
      ),
    );
  }
}


