const functions = require("firebase-functions");
const stripe = require("stripe")(functions.config().stripe.testkey); //terminal: firebase functions:config:set stripe.testkey="sk_test_51..."
const admin = require('firebase-admin');
admin.initializeApp();
//firebase deploy --only functions

// Create and deploy your first functions
// https://firebase.google.com/docs/functions/get-started


// Takes init payment request from a customer in app and returns the payment intent
// Payment code copied from https://www.youtube.com/watch?v=BR4sF_VzV0w
// github: https://github.com/hadikachmar3/grocery_app_course/blob/master/functions/index.js
exports.stripePaymentIntentRequest = functions
    .region("europe-west1")
    .https.onRequest(async (req, res) => {
      try {
        let customerId;

        // Gets the customer who's email id matches the one sent by the client
        const customerList = await stripe.customers.list({
          email: req.body.email,
          limit: 1,
        });

        // Checks the if the customer exists, if not creates a new customer
        if (customerList.data.length !== 0) {
          customerId = customerList.data[0].id;
        } else {
          const customer = await stripe.customers.create({
            email: req.body.email,
          });
          customerId = customer.data.id;
        }

        // Creates a temporary secret key linked with the customer
        const ephemeralKey = await stripe.ephemeralKeys.create(
            {customer: customerId},
            {apiVersion: "2020-08-27"},
        );

        // Creates a new payment intent with amount passed in from the client
        const paymentIntent = await stripe.paymentIntents.create({
          amount: parseInt(req.body.amount),
          currency: "nok",
          customer: customerId,
        });

        res.status(200).send({
          paymentIntent: paymentIntent.client_secret,
          ephemeralKey: ephemeralKey.secret,
          customer: customerId,
          success: true,
        });
      } catch (error) {
        res.status(404).send({success: false, error: error.message});
      }
    });

/*
exports.stripePaymentSuccess = functions
    .region("europe-west1")
    .https.onRequest(async (req, res) => {
      var db = admin.database()
      var ref = db.ref
    });
*/