import 'package:billettapp_for_vitensenteret/programmingGame/programmingGame.dart';
import 'package:billettapp_for_vitensenteret/towerOfHanoi/functions.dart';
import 'package:billettapp_for_vitensenteret/towerOfHanoi/game_object.dart';
import 'package:billettapp_for_vitensenteret/wordle/models/letter_model.dart';
import 'package:billettapp_for_vitensenteret/wordle/models/word_model.dart';
import 'package:billettapp_for_vitensenteret/wordle/views/wordle_screen.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  //First we test the functions inside the programming app
  group('Programming Game (Viten Kode)', () {
    test(
        'compare two lists, should check solution with current guess from the user, they match', () {
      final result = checkWin(["A", "B", "C"], ["A", "B", "C"]);

      expect(result, true);
    });

    test(
        'compare two lists, should check solution with current guess from the user, they dont match', () {
      final result = checkWin(["A", "B", "C"], ["A", "C", "B"]);

      expect(result, false);
    });

    test(
        'compare two lists, should check solution with current guess from the user, user wrote random values', () {
      final result = checkWin(["A", "B", "C"], [
        "12457rtfdsa",
        "Bdafgvjbwqd",
        "Csdatfgu12563297dhkegacjkbdcgkueygdghdsg"
      ]);

      expect(result, false);
    });
  });
  group('Tower of Hanoi Game', () {
      test('Function checks rules to see if the piece is allowed to move, move to unocupied space', () {
        final testGameObject = GameObject(
            color: "red", lastIndex: 0, imageId: "assets/images/red.png");
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];


        final result = checkIfPieceCanBeMoved(testGameObject, testgameboard, 7);

        expect(result, true);
      });

      test('Function checks rules to see if the piece is allowed to move, move to space where a piece already is', () {
        final testGameObject = GameObject(
            color: "yellow", lastIndex: 0, imageId: "assets/images/red.png");
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'red',
          'empty',
          'empty',
          'empty',
          'empty'
        ];


        final result = checkIfPieceCanBeMoved(testGameObject, testgameboard, 8);

        expect(result, false);
      });

      test('Function checks rules to see if the piece is allowed to move, try to put bigger piece ontop of smaller piece', () {
        final testGameObject = GameObject(
            color: "yellow", lastIndex: 0, imageId: "assets/images/red.png");
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'red',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];


        final result = checkIfPieceCanBeMoved(testGameObject, testgameboard, 8);

        expect(result, false);
      });

      test('Function checks if the location the player is trying to place a piece on has anything under it, if it can be placed, it will return true, move to unocupied space', () {
        final testGameObject = GameObject(
            color: "red", lastIndex: 0, imageId: "assets/images/red.png");
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];


        final result = checkLegalMove(testGameObject, testgameboard, 7, 0);

        expect(result, true);
      });

      test('Function checks if the location the player is trying to place a piece on has anything under it, if it can be placed, it will return true, move ontop of bigger piece', () {
        final testGameObject = GameObject(
            color: "red", lastIndex: 0, imageId: "assets/images/red.png");
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];


        final result = checkLegalMove(testGameObject, testgameboard, 8, 0);

        expect(result, true);
      });

      test('Function checks if the location the player is trying to place a piece on has anything under it, if it can be placed, it will return true, move ontop of smaller piece', () {
        final testGameObject = GameObject(
            color: "yellow", lastIndex: 0, imageId: "assets/images/red.png");
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'red',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];


        final result = checkLegalMove(testGameObject, testgameboard, 7, 0);

        expect(result, false);
      });

      test('Function checks if the board meets the win condition, sending winning board', () {
        List<String> testgameboard = ['empty', 'empty','empty','empty','empty','empty','empty','empty','red', 'green', 'blue', 'yellow', 'empty'];

        final result = checkWinCondition(testgameboard);

        expect(result, true);
      });

      test('Function checks if the board meets the win condition, sending wrong board', () {
        List<String> testgameboard = ['empty','empty','red', 'green', 'blue', 'yellow', 'empty', 'empty', 'empty','empty','empty','empty','empty'];

        final result = checkWinCondition(testgameboard);

        expect(result, false);
      });

      test('Function checks if the board meets the win condition, sending jumbled short board', () {
        List<String> testgameboard = ['fygacwbhsd','ygw7ey782t83ggyCF'];

        final result = checkWinCondition(testgameboard);

        expect(result, false);
      });

      test('Function that returns true if the current index being checked is occupied by a gamepiece, checking empty slot', () {
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'red',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = checkIfOccupied(testgameboard, 6);

        expect(result, false);
      });

      test('Function that returns true if the current index being checked is occupied by a gamepiece, checking non-empty slot', () {
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'red',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = checkIfOccupied(testgameboard, 3);

        expect(result, true);
      });

      test('Function that returns true if the current index being checked is occupied by a gamepiece, checking outside array', () {
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'red',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = checkIfOccupied(testgameboard, 37);

        expect(result, false);
      });

      test('Checks the given index if a the gamepiece can be placed there based on the color of the piece, red can always be placed', () {
        final testGameObject = GameObject(
            color: "red", lastIndex: 0, imageId: "assets/images/red.png");
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = canPieceBePlaced(testGameObject, testgameboard, 7);

        expect(result, true);
      });

      test('Checks the given index if a the gamepiece can be placed there based on the color of the piece, red placed ontop of yellow', () {
        final testGameObject = GameObject(
            color: "red", lastIndex: 0, imageId: "assets/images/red.png");
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = canPieceBePlaced(testGameObject, testgameboard, 6);

        expect(result, true);
      });

      test('Checks the given index if a the gamepiece can be placed there based on the color of the piece, blue on top of red is not legal', () {
        final testGameObject = GameObject(
            color: "blue", lastIndex: 0, imageId: "assets/images/red.png");
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'red',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = canPieceBePlaced(testGameObject, testgameboard, 6);

        expect(result, false);
      });

      test('function counts the number of empty places under the chosen index, returning next available index, counting from 4, should return 7', () {
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = indexesToPieceUnderneath(testgameboard, 4);

        expect(result, 7);
      });

      test('function counts the number of empty places under the chosen index, returning next available index, counting from 4, should return 5', () {
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'red',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = indexesToPieceUnderneath(testgameboard, 4);

        expect(result, 5);
      });

      test('function counts the number of empty places under the chosen index, returning next available index, counting from 11, should return 11', () {
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = indexesToPieceUnderneath(testgameboard, 11);

        expect(result, 11);
      });

      test('function returns the id of the gamepiece at the requested tile, requesting at 0, which has red', () {
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = getId(testgameboard, 0);

        expect(result, 0);
      });

      test('function returns the id of the gamepiece at the requested tile, requesting at 2, which has blue', () {
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = getId(testgameboard, 2);

        expect(result, 2);
      });

      test('function returns the id of the gamepiece at the requested tile, requesting at 7, which has empty', () {
        List<String> testgameboard = [
          'red',
          'green',
          'blue',
          'yellow',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty',
          'empty'
        ];

        final result = getId(testgameboard, 0);

        expect(result, 0);
      });

      test('Function takes the tileId and returns which image should be displayed, getting top of a rod', () {
        final result = getImage(0);

        expect(result, 'images/rodTop.png');
      });

      test('Function takes the tileId and returns which image should be displayed, getting middle of a rod', () {
        final result = getImage(5);

        expect(result, 'images/rodCenter.png');
      });

      test('Function takes the tileId and returns which image should be displayed, getting outside array', () {
        final result = getImage(13);

        expect(result, 'images/rodCenter.png');
      });

      test('Function displays victory text based on number of moves used, sending best result', () {
        final result = victoryMessageBasedOnMoves(15);

        expect(result, 'Gratulerer, du klarte det med minst antall trekk!');
      });

      test('Function displays victory text based on number of moves used, sending decent result', () {
        final result = victoryMessageBasedOnMoves(25);

        expect(result, 'Gratulerer, du klarte det!');
      });

      test('Function displays victory text based on number of moves used, sending high number', () {
        final result = victoryMessageBasedOnMoves(57634);

        expect(result, 'Imponerende mengde trekk. Du gav deg aldri, gratulerer!');
      });
    });
  group('Word Game (Viten Ord)', () {
    test('Function returns the index where the instance of the letter was located at, checking letter in word', () {
      Word solution = Word.fromString("ABCDE");
      Letter guess = Letter(val: "B");


      final result = firstIndexFoundAt(solution.letters, guess);

      expect(result, 1);
    });

    test('Function returns the index where the instance of the letter was located at, checking letter with multiple instances in word', () {
      Word solution = Word.fromString("ABCBE");
      Letter guess = Letter(val: "B");


      final result = firstIndexFoundAt(solution.letters, guess);

      expect(result, 1);
    });
    test('Function returns the index where the instance of the letter was located at, checking something that isnt in word', () {
      Word solution = Word.fromString("ABCDE");
      Letter guess = Letter(val: "G");


      final result = firstIndexFoundAt(solution.letters, guess);

      expect(result, 10);
    });

    test('Function counts the number of instances of an item in the list, and if the number excedes 1, then returns true. Check word with multiple instances', () {
      Word solution = Word.fromString("ABBDE");
      Letter guess = Letter(val: "B");


      final result = guessContainsMultipleLetters(solution.letters, guess);

      expect(result, true);
    });

    test('Function counts the number of instances of an item in the list, and if the number excedes 1, then returns true. Check word without multiple instances', () {
      Word solution = Word.fromString("ABCDE");
      Letter guess = Letter(val: "B");


      final result = guessContainsMultipleLetters(solution.letters, guess);

      expect(result, true);
    });

    test('Function counts the number of instances of an item in the list, and if the number excedes 1, then returns true. Check word without letter', () {
      Word solution = Word.fromString("ABBDE");
      Letter guess = Letter(val: "Y");


      final result = guessContainsMultipleLetters(solution.letters, guess);

      expect(result, false);
    });


  });
}